﻿Imports System.Net.Mail
Public Class MailDaten

    Private _MailSettingsRow As DtsDaten.EmailAdresseRow
    Private _SenderAddress As String
    Private _Password As String
    Private _Port As Integer
    Private _SMTPServer As String
    Private _SSL As Boolean
    Private _Recipient As String
    Private _BCC As String
    Private _CC As String
    Private _Subject As String
    Private _Message As String
    Private _Signature As String
    Private _Attachments As List(Of String)
    Private _MailCredentials As New System.Net.NetworkCredential

    Public Sub New(MailSettingsRow As DtsDaten.EmailAdresseRow, Recipient As String, bcc As String, cc As String, Subject As String, Message As String, Signature As String, Attachments As List(Of String))
        _MailSettingsRow = MailSettingsRow
        _Recipient = Recipient
        _BCC = bcc
        _CC = cc
        _Subject = Subject
        _Message = Message
        _SenderAddress = _MailSettingsRow.Adresse
        _Password = _MailSettingsRow.Passwort
        _Port = _MailSettingsRow.Port
        _SMTPServer = _MailSettingsRow.SMTPServer
        _SSL = _MailSettingsRow.SSL
        _Attachments = Attachments
        _Signature = Signature
    End Sub
    Public Function CreateMail() As MailMessage
        Dim Mail As New MailMessage
        Mail.IsBodyHtml = False
        With Mail
            .From = New MailAddress(_SenderAddress)
            .To.Add(_Recipient)
            If _BCC <> "" Then .Bcc.Add(_BCC)
            If _CC <> "" Then .CC.Add(_CC)
            .Subject = _Subject
            .Body = _Message
            If _Attachments.Count > 0 Then
                For i = 0 To _Attachments.Count - 1
                    .Attachments.Add(New System.Net.Mail.Attachment(_Attachments(i)))
                Next
            End If
        End With
        Return Mail
    End Function
    Public Function CreateSMTPClient() As SmtpClient
        Dim SMTP As New SmtpClient()
        With SMTP
            .EnableSsl = _SSL
            .Host = _SMTPServer
            .Port = _Port
            .UseDefaultCredentials = False
            .Credentials = _MailCredentials
        End With
        Return SMTP
    End Function
    Public Function CreateMailCredentials() As System.Net.NetworkCredential
        _MailCredentials.UserName = _SenderAddress
        _MailCredentials.Password = _Password
        Return _MailCredentials
    End Function
    Public Sub AddSignatureToMessage()
        If _Signature.IsNotEmpty Then _Message &= Environment.NewLine & Environment.NewLine & _Signature
    End Sub
    Public Function CheckForValidMailAddresses() As Tuple(Of Boolean, String)
        If _Recipient.NotContains("@") OrElse _Recipient.NotContains(".") Then
            Return New Tuple(Of Boolean, String)(False, "ungültige Empfängeradresse")
        End If
        If _BCC.IsNotEmpty Then
            If _BCC.NotContains("@") OrElse _BCC.NotContains(".") Then
                Return New Tuple(Of Boolean, String)(False, "ungültige bcc Adresse")
            End If
        End If
        If _CC.IsNotEmpty Then
            If _CC.NotContains("@") OrElse _CC.NotContains(".") Then
                Return New Tuple(Of Boolean, String)(False, "ungültige cc Adresse")
            End If
        End If
        Return New Tuple(Of Boolean, String)(True, "")
    End Function
    Public Function CheckForMailText() As (SubjectHasText As Boolean, SubjectErrorText As String, MessageHasText As Boolean, MessageErrorText As String)
        Dim SubjectHasText = _Subject.IsNotEmpty
        Dim MessageHasText = _Message.IsNotEmpty
        Return (SubjectHasText, "kein Betreff", MessageHasText, "keine Nachricht")
    End Function
End Class
