﻿Public Class UCLieferungen
    Public Event OpenCustomerOrderFound(BSCurrent As Object)
    Private _SaveDGVColumnWidths As Boolean
    Private Const _ColumnThatContainsOrderDate As Integer = 0
    Private Const _ColumnThatContainsDeliveryDate As Integer = 1
    Private Const _ColumnThatContainsDateOfInvoiceReceive As Integer = 2
    Private Const _ColumnThatContainsInvoiceNumber As Integer = 4
    Private Sub DGVLieferungen_LostFocus(sender As Object, e As EventArgs) Handles DGVLieferungen.LostFocus
        _SaveDGVColumnWidths = False
    End Sub
    Private Sub DGVLieferungen_ColumnWidthChanged(sender As Object, e As DataGridViewColumnEventArgs) Handles DGVLieferungen.ColumnWidthChanged
        If _SaveDGVColumnWidths Then My.Settings.DGVLieferungenSpaltenBreite = GetDGVColumnWidthsAsSaveableString(DGVLieferungen)
    End Sub
    Private Sub DGVLieferungen_Click(sender As Object, e As EventArgs) Handles DGVLieferungen.Click
        _SaveDGVColumnWidths = True
    End Sub
    Private Sub BTN_Click(sender As Object, e As EventArgs) Handles BTNNeueLieferung.Click, BTNLieferungLoeschen.Click, BTNBestellprogrammStarten.Click,
            BTNUmsatzauswertung.Click, BTNLieferungSpeichern.Click, BTNLieferungmitBstProgramm.Click
        Select Case True
            Case sender Is BTNNeueLieferung : CreateNewOrder()
            Case sender Is BTNLieferungmitBstProgramm : CreateNewOrderAndStartOrderProgram()
            Case sender Is BTNLieferungLoeschen : DeleteOrder()
            Case sender Is BTNBestellprogrammStarten : StartOrderProgram()
            Case sender Is BTNUmsatzauswertung : ShowSalesEvaluation()
            Case sender Is BTNLieferungSpeichern : CheckUserInputAndSaveOrder()
        End Select
    End Sub
    Private Sub DGVLieferungen_CellDoubleClick(sender As Object, e As DataGridViewCellEventArgs) Handles DGVLieferungen.CellDoubleClick
        Dim SelectedOrder = BSLieferant.At(Of DtsDaten.LieferungRow)
        Dim DoubleClickedColumnIndex = e.ColumnIndex

        Select Case DoubleClickedColumnIndex
            Case = _ColumnThatContainsDeliveryDate
                If SelectedOrder.IsLieferdatumNull Then SelectedOrder.Lieferdatum = System.DateTime.Now
            Case = _ColumnThatContainsDateOfInvoiceReceive
                If SelectedOrder.IsRechnungserhaltNull Then SelectedOrder.Rechnungserhalt = System.DateTime.Now
            Case = _ColumnThatContainsOrderDate : LoadOrderFile()
            Case = _ColumnThatContainsInvoiceNumber : LoadInvoice()
        End Select
    End Sub
    Private Sub BSLieferant_CurrentChanged(sender As Object, e As EventArgs) Handles BSLieferant.CurrentChanged
        CalculateAndWriteSales()
    End Sub
    Private Sub CreateNewOrder()
        Dim SelectedSupplier = BSLieferant.At(Of DtsDaten.LieferantRow)
        Dim Dts = DirectCast(BSMain.DataSource, DtsDaten)

        Dim NewOrder As DtsDaten.LieferungRow
        NewOrder = Dts.Lieferung.NewLieferungRow()
        NewOrder.Bestelldatum = System.DateTime.Now
        NewOrder.LieferantID = SelectedSupplier.ID
        Dts.Lieferung.Rows.Add(NewOrder)
        BSLieferung.MoveLast()
    End Sub
    Private Sub DeleteOrder()
        If BSLieferung.Current Is Nothing Then Return
        Dim SelectedOrder = BSLieferung.At(Of DtsDaten.LieferungRow)
        If ShowMessageBoxAndAskUserYesOrNo($"Die Bestellung vom {SelectedOrder.Bestelldatum.ToString("dd.MM.yyyy")} wird gelöscht.
                Sind sie sicher?") Then BSLieferung.RemoveCurrent()
    End Sub
    Private Sub LoadOrderFile()
        ShowMessageBox("Bestelldatei anzeigen")
    End Sub
    Private Sub LoadInvoice()
        ShowMessageBox("Rechnung anzeigen")
    End Sub
    Private Sub StartOrderProgram()
        ShowMessageBox("Dieser Button startet nur das Bestellprogramm")
    End Sub
    Private Sub ShowSalesEvaluation()
        ShowMessageBox("Dieser Button startet die Umsatzauswertung")
    End Sub
    Private Sub CreateNewOrderAndStartOrderProgram()
        ShowMessageBox("Dieser Button legt eine neue Bestellung an und startet das Bestellprogramm")
    End Sub
    Private Sub CheckUserInputAndSaveOrder()
        Dim UserInput = CheckUserInput()
        If Not UserInput.UserInputCorret Then
            ShowAutoClosingMessageBox(UserInput.ErrorText)
            Return
        End If
        SaveOrder()
        CalculateAndWriteSales()
        CheckForCustomerOrders()
    End Sub
    Private Sub CheckForCustomerOrders()
        If OpenCostumerOrdersPresent() Then
            If ShowMessageBoxAndAskUserYesOrNo("Es wurden offene Kundenbestellungen gefunden." & Environment.NewLine &
                         "sollen diese angezeigt werden?") Then
                RaiseEvent OpenCustomerOrderFound(BSLieferant.Current)
            End If
        End If
    End Sub
    Private Function CheckUserInput() As (UserInputCorret As Boolean, ErrorText As String)
        If BSLieferung.Current Is Nothing Then Return (True, "Bitte erst eine Bestellung anlegen")

        Dim net19 As Double
        If Not Double.TryParse(TBNet19.Text, net19) Then Return (False, "ungültige Netto 19% Summe")
        Dim net7 As Double
        If Not Double.TryParse(TBNet7.Text, net7) Then Return (False, "ungültige Netto 7% Summe")

        Return (True, "")
    End Function
    Private Function OpenCostumerOrdersPresent() As Boolean
        Dim SelectedSupplier = BSLieferant.At(Of DtsDaten.LieferantRow)
        Dim OpenCustomerOrders = SelectedSupplier.GetKundenbestellungRows.FirstOrDefault(Function(z) Not z.IsDatumBestelltNull AndAlso z.IsDatumAngerufenNull)
        Return OpenCustomerOrders IsNot Nothing
    End Function
    Private Sub CalculateAndWriteSales()
        Dim ThisYear = System.DateTime.Now.Year
        Dim LastYear = ThisYear - 1
        Dim SalesThisYear = CalculateYearlySales(ThisYear)
        Dim SalesLastYear = CalculateYearlySales(LastYear)
        WriteSales(SalesThisYear, SalesLastYear, ThisYear, LastYear)
    End Sub
    Private Function CalculateYearlySales(year As Integer) As Double
        Dim SelectedSupplier = BSLieferant.At(Of DtsDaten.LieferantRow)
        Dim YearlyOrders = SelectedSupplier.GetLieferungRows.Where(Function(z) Not z.IsRechnungsdatumNull AndAlso z.Rechnungsdatum.Year = year)

        Dim TotalAmount As Double
        For Each order In YearlyOrders
            TotalAmount += order.Netto19 + order.Netto7
        Next
        Return TotalAmount
    End Function
    Private Sub WriteSales(SalesThisYear As Double, SalesLastYear As Double, ThisYear As Integer, LastYear As Integer)
        LBLUmsatzDiesesJahrText.Text = $"Umsatz {ThisYear}"
        LBLUmsatzDiesesJahr.Text = SalesThisYear.ToString("#,##0.00 €")
        LBLUmsatzLetztesJahrText.Text = $"Umsatz {LastYear}"
        LBLUmsatzLetztesJahr.Text = SalesLastYear.ToString("#,##0.00 €")
    End Sub
    Private Sub SaveOrder()
        Dim SelectedOrder = BSLieferung.At(Of DtsDaten.LieferungRow)
        Dim SelectedSupplier = BSLieferant.At(Of DtsDaten.LieferantRow)
        Dim net19 As Double = Double.Parse(TBNet19.Text)
        Dim net7 As Double = Double.Parse(TBNet7.Text)

        Dim TotalInvoiceAmount As Double
        If SelectedSupplier.MwstAusrechnen Then
            TotalInvoiceAmount = (net19 * 1.19) + (net7 * 1.07)
        Else
            TotalInvoiceAmount = net19 + net7
        End If

        With SelectedOrder
            .Bestelldatum = DTPBestelldatum.Value
            If NDTPLieferdatum.Checked Then
                .Lieferdatum = CDate(NDTPLieferdatum.Value)
                .Lieferzeit = CDate(NDTPLieferdatum.Value).Subtract(DTPBestelldatum.Value).TotalDays
            Else
                .SetLieferdatumNull()
                .SetLieferzeitNull()
            End If
            If NDTPRechnungserhalt.Checked Then
                .Rechnungserhalt = CDate(NDTPRechnungserhalt.Value)
            Else
                .SetRechnungserhaltNull()
            End If
            If NDTPRechnungsdatum.Checked Then
                .Rechnungsdatum = CDate(NDTPRechnungsdatum.Value)
            Else
                .SetRechnungsdatumNull()
            End If
            .Rechnungsnummer = TBRechnungsnummer.Text
            .Netto19 = net19
            .Netto7 = net7
            .Gesamt = TotalInvoiceAmount
            .Notiz = TBNotiz.Text
        End With
    End Sub
End Class
